/*******************************************************************************
*  emeter-app.h -
*
*  Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/ 
* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*
*    Redistributions of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************/

/*! \file */

#if !defined(_METER_APP_H_)
#define _METER_APP_H_

/*! \brief Initialise all the data and peripherals, and prepare the machine to run
after reset. */
void system_setup(void);

#define USE_WATCHDOG

#define disable_watchdog()             (WDTCTL = (WDTCTL & 0xFF) | WDTPW | WDTHOLD) //WDTCTL =  WDT_ARST_1000

#if defined(__MSP430__)  &&  defined(USE_WATCHDOG)
#define kick_watchdog()             (WDTCTL = (WDTCTL & 0xFF) | WDTPW | WDTHOLD) //WDTCTL =  WDT_ARST_1000
#else
#define kick_watchdog()             /**/
#endif


/*! This structure is an image of the non-volatile calibration parameters
stored in the info memory. */
struct nv_parms_s
{ 
  /*! \brief RTC offset unit 1ppm*/
  int16_t crystal_offset_ppm;  
};

/*! This union overlays a segment of the info, ensuring correct
alignment. */  
struct info_mem_s
{
  union
  {
    struct nv_parms_s s;
    int8_t x[128];
  } seg_a;
};

extern const struct info_mem_s nv_parms;
#endif
