/*******************************************************************************
*  emeter-setup.c -
*
*  Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/ 
* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*
*    Redistributions of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************/

/*! \file emeter-structs.h */

//  MSP430 setup routines for e-meters.
//
//  This software is appropriate for single phase and three phase e-meters
//  using a voltage sensor plus CT or shunt resistor current sensors, or
//  a combination of a CT plus a shunt. 
//
#include <stdint.h>
#include <string.h>
#include <io.h>

#include "emeter-template.h"
#include "emeter-app.h"
#include "emeter-dlt645.h"
#include "hal_pmm.h"

#if defined(__MSP430__)
void system_setup(void)
{
	SetVCore(3);

#if defined(__MSP430_HAS_UCS__)
  __bis_SR_register(SCG0);                        /* Disable the FLL control loop */
  UCSCTL6 = (UCSCTL6 | XT1DRIVE_3);               /* Highest drive setting for XT1 startup */
	UCSCTL6 &= ~(XCAP0_L|XCAP1_L);
  while (SFRIFG1 & OFIFG)
  {
    /* Check OFIFG fault flag */
    UCSCTL7 &= ~(DCOFFG | XT1LFOFFG | XT2OFFG); /* Clear OSC fault flags */
    SFRIFG1 &= ~OFIFG;                          /* Clear OFIFG fault flag */
  }
  UCSCTL6 = (UCSCTL6 & ~(XT1DRIVE_3));            /* Reduce drive to something weaker */
  
  UCSCTL1 = DCORSEL_5; //DCORSEL_5;                            /* Set RSELx for DCO = 16 MHz */
  UCSCTL2 = FLLD_1 + 255;                         /* Set DCO Multiplier for 8MHz */
  /* (N + 1) * FLLRef = Fdco */
  /* (255 + 1) * 32768 = 8MHz */
  /* Set FLL Div = fDCOCLK/2 */
  __bic_SR_register(SCG0);                        /* Enable the FLL control loop */
  
  UCSCTL5 |= DIVS__4;                             // 16/4 SMCLK
  UCSCTL4 = SELM__DCOCLK | SELS__DCOCLK | SELA__XT1CLK;     //16MHz MCLK, 4MHz SMCLK, 32KHz ACLK

	/* Suitable settings for a 6736 */
	//PJDIR = BIT0 | BIT1 | BIT3; 		/* SMCLK, MCLK, ACLK */
	//PJSEL = BIT0 | BIT1 | BIT3; 		/* SMCLK, MCLK, ACLK */


#if 0  
	while (1)
	{
		__delay_cycles(10000);
		__delay_cycles(10000);		
	}
#endif  

#endif
  
#if defined(P1OUT_INIT)
  P1OUT = P1OUT_INIT;
#endif
#if defined(P1DIR_INIT)
  P1DIR = P1DIR_INIT;
#endif
#if defined(P1SEL_INIT)
  P1SEL = P1SEL_INIT;
#endif
  
#if defined(P2OUT_INIT)
  P2OUT = P2OUT_INIT;
#endif
#if defined(P2DIR_INIT)
  P2DIR = P2DIR_INIT;
#endif
#if defined(P2SEL_INIT)
  P2SEL = P2SEL_INIT;
#endif
  
#if defined(P3OUT_INIT)
  P3OUT = P3OUT_INIT;
#endif
#if defined(P3DIR_INIT)
  P3DIR = P3DIR_INIT;
#endif
#if defined(P3SEL_INIT)
  P3SEL = P3SEL_INIT;
#endif
  
#if defined(P4OUT_INIT)
  P4OUT = P4OUT_INIT;
#endif
#if defined(P4DIR_INIT)
  P4DIR = P4DIR_INIT;
#endif
#if defined(P4SEL_INIT)
  P4SEL = P4SEL_INIT;
#endif
  
#if defined(P5OUT_INIT)
  P5OUT = P5OUT_INIT;
#endif
#if defined(P5DIR_INIT)
  P5DIR = P5DIR_INIT;
#endif
#if defined(P5SEL_INIT)
  P5SEL = P5SEL_INIT;
#endif
  
#if defined(P6OUT_INIT)
  P6OUT = P6OUT_INIT;
#endif
#if defined(P6DIR_INIT)
  P6DIR = P6DIR_INIT;
#endif
#if defined(P6SEL_INIT)
  P6SEL = P6SEL_INIT;
#endif
  
#if defined(P7OUT_INIT)
  P7OUT = P7OUT_INIT;
#endif
#if defined(P7DIR_INIT)
  P7DIR = P7DIR_INIT;
#endif
#if defined(P7SEL_INIT)
  P7SEL = P7SEL_INIT;
#endif
  
#if defined(P8OUT_INIT)
  P8OUT = P8OUT_INIT;
#endif
#if defined(P8DIR_INIT)
  P8DIR = P8DIR_INIT;
#endif
#if defined(P8SEL_INIT)
  P8SEL = P8SEL_INIT;
#endif
  
#if defined(P9OUT_INIT)
  P9OUT = P9OUT_INIT;
#endif
#if defined(P9DIR_INIT)
  P9DIR = P9DIR_INIT;
#endif
#if defined(P9SEL_INIT)
  P9SEL = P9SEL_INIT;
#endif
  
#if defined(P10OUT_INIT)
  P10OUT = P10OUT_INIT;
#endif
#if defined(P10DIR_INIT)
  P10DIR = P10DIR_INIT;
#endif
#if defined(P10SEL_INIT)
  P10SEL = P10SEL_INIT;
#endif

#if defined(PJOUT_INIT)
		PJOUT = PJOUT_INIT;
#endif
#if defined(PJDIR_INIT)
		PJDIR = PJDIR_INIT;
#endif
#if defined(PJSEL_INIT)
		PJSEL = PJSEL_INIT;
#endif

}
#else
void system_setup(void)
{
  /* Prime the DC estimates for quick settling */
#if defined(NEUTRAL_MONITOR_SUPPORT)
  dc_filter_current_init(phase->metrology.neutral.I_dc_estimate, phase_nv->current.initial_dc_estimate);
#endif
  dc_filter_current_init(phase->metrology.current.I_dc_estimate, phase_nv->current.initial_dc_estimate);
  dc_filter_voltage_init(phase->metrology.V_dc_estimate[0], phase_nv->initial_v_dc_estimate[0]);
  //phase->metrology.V_dc_estimate[1] = phase_nv->initial_v_limp_dc_estimate;
  phase->metrology.mains_period = ((SAMPLES_PER_10_SECONDS*6554)/MAINS_NOMINAL_FREQUENCY) << 8;
  
  samples_per_second = SAMPLES_PER_10_SECONDS/10;
#if defined(TEMPERATURE_SUPPORT)
  temperature = 0;
#endif
}
#endif


