/*******************************************************************************
 *  emeter-template.h -
 *
 *  Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

/* This switch enables support of an IR receiver and/or transmitter for
   programming and/or reading the meter. */
#define SERIAL_COMMS_SUPPORT
#define DLT645_SUPPORT

/*! This defines the speed of USART 0 or USCI 0 */
#define UART_PORT_0_SUPPORT                                 0
#define UART_PORT_1_SUPPORT                                 0
#define UART_PORT_2_SUPPORT                                 1

#define UART_PORT_0_SN 			(UART_PORT_0_SUPPORT-1)
#define UART_PORT_1_SN 			(UART_PORT_0_SUPPORT+UART_PORT_1_SUPPORT-1)
#define UART_PORT_2_SN 			(UART_PORT_0_SUPPORT+UART_PORT_1_SUPPORT+UART_PORT_2_SUPPORT-1)


#define UART2_BAUD_RATE                                     9600
#define UART2_DLT645_SUPPORT                                1

//-------------------------------------------Io configuration---------------------------------------------------

#if 1
/*
    P1.0 = Temperature input ADC
    P1.1 = Battery input ADC
    P1.2 = UART 0 Rx IrDA
    P1.3 = UART 0 Tx IrDA			(output)
    P1.4 = UART 1 Rx RS232C
    P1.5 = UART 1 Tx RS232C	(output)
    P1.6 = Check relay input
    P1.7 = STA input
 */
#define P1DIR_INIT                                  (BIT5 | BIT3)
#define P1SEL_INIT                                  (BIT5 | BIT4 | BIT3 | BIT2| BIT1 | BIT0)
#define P1OUT_INIT                                  (0)

/*
    P2.0 = SCL (software simulation)
    P2.1 = SDA
    P2.2 = 485 txd
    P2.3 = 485 rxd
    P2.4 = Check open cover 
    P2.5 = Key prog
    P2.6 = Key disp 
    P2.7 = 38k pwm
 */
#define P2DIR_INIT                                  (BIT7 | BIT3)
#define P2SEL_INIT                                  (BIT3 | BIT2) //(BIT7 | BIT3 | BIT2)
#define P2OUT_INIT                                  (0)

/*
    P3.0 = IC-STAT  (output)
    P3.1 = IC-CLK (output)
    P3.2 = IR-RXD
    P3.3 = IR-TXD
    P3.4 = POW-CTL (output)
    P3.5 = LCD segment line 32
    P3.6 = LCD segment line 31
    P3.7 = LCD segment line 30
 */
#define P3DIR_INIT                                  (BIT4 | BIT1 | BIT0)
#define P3SEL_INIT                                  (BIT3 | BIT2)
#define P3OUT_INIT                                  (0)

/*
    P4.0 = LCD segment line 29
    P4.1 = LCD segment line 28
    P4.2 = LCD segment line 27
    P4.3 = LCD segment line 26
    P4.4 = LCD segment line 25
    P4.5 = LCD segment line 24
    P4.6 = LCD segment line 23
    P4.7 = LCD segment line 22
 */
#define P4DIR_INIT                                  0
#define P4SEL_INIT                                  0
#define P4OUT_INIT                                  0

/*
    P5.0 = LCD segment line 21
    P5.1 = LCD segment line 20
    P5.2 = LCD segment line 19
    P5.3 = LCD segment line 18
    P5.4 = LCD segment line 17
    P5.5 = LCD segment line 16
    P5.6 = LCD segment line 15
    P5.7 = LCD segment line 14
*/
#define P5DIR_INIT                                  0
#define P5SEL_INIT                                  0
#define P5OUT_INIT                                  0

/*
    P6.0 = LCD segment line 13
    P6.1 = LCD segment line 12
    P6.2 = LCD segment line 11
    P6.3 = LCD segment line 10
    P6.4 = LCD segment line 9
    P6.5 = LCD segment line 8
    P6.6 = LCD segment line 7
    P6.7 = LCD segment line 6
*/
#define P5DIR_INIT                                  0
#define P5SEL_INIT                                  0
#define P5OUT_INIT                                  0

/*
    P7.0 = LCD segment line 5
    P7.1 = LCD segment line 4
    P7.2 = LCD segment line 3
    P7.3 = LCD segment line 2
    P7.4 = LCD segment line 1
    P7.5 = LCD segment line 0
    P7.6 = Evenout
    P7.7 = /RST
 */
#define P7DIR_INIT                                  (BIT6 | BIT7)
#define P7SEL_INIT                                  0
#define P7OUT_INIT                                  0

/*
    P8.0 = RL-OFF
    P8.1 = RL-ON
    P8.2 = /SET
    P8.3 = CF-ACT
    P8.4 = IC-RST
    P8.5 = E-RST
    P8.6 = E-DATA
    P8.7 = IC-DATA
 */
#define P8DIR_INIT                                  (BIT7|BIT6|BIT5|BIT4|BIT3|BIT2|BIT1|BIT0)
#define P8SEL_INIT                                  (0)
#define P8OUT_INIT                                  (0)

/*
    P9.0 = BELL
    P9.1 = NA
    P9.2 = NA
    P9.3 = NA
    P9.4 = 
    P9.5 = 
    P9.6 = 
    P9.7 = 
 */
#define P9DIR_INIT                                  (BIT3|BIT2|BIT1|BIT0)
#define P9SEL_INIT                                  (0)
#define P9OUT_INIT                                  (0)

/*
    PJ.0 = E-CLK
    PJ.1 = ALARM
    PJ.2 = LIGHT
    PJ.3 = VRTC
    PJ.4 = 
    PJ.5 = 
    PJ.6 = 
    PJ.7 = 
 */
#define PJDIR_INIT                                  (BIT3|BIT2|BIT1|BIT0)
#define PJSEL_INIT                                  (0)
#define PJOUT_INIT                                  (0)
#else
/*
    P1.0 = Voltage sense 1
    P1.1 = Voltage sense 2
    P1.2 = UART 0 Rx IrDA
    P1.3 = UART 0 Tx IrDA
    P1.4 = UART 1 Rx RS232C
    P1.5 = UART 1 Tx RS232C
    P1.6 = Active pulse
    P1.7 = Reactive pulse
 */
#define P1DIR_INIT                                  (BIT7 | BIT6 | BIT5 | BIT3)
#define P1SEL_INIT                                  (BIT5 | BIT4 | BIT3 | BIT2)
#define P1OUT_INIT                                  (BIT7 | BIT6)

/*
    P2.0 = SCL
    P2.1 = SDA
    P2.2 =
    P2.3 =
    P2.4 = 
    P2.5 = 
    P2.6 = Button 1 
    P2.7 = Button 2
 */
#define P2DIR_INIT                                  (BIT3)
#define P2SEL_INIT                                  (BIT2|BIT3)
#define P2OUT_INIT                                  (0)

/*
    P3.0 = LED 3
    P3.1 = IrDA SD
    P3.2 = RF GPIO2
    P3.3 = RF GPIO1
    P3.4 = LCD segment line 39
    P3.5 = LCD segment line 38
    P3.6 = LCD segment line 37
    P3.7 = LCD segment line 36
 */
#define P3DIR_INIT                                  (BIT0)
#define P3SEL_INIT                                  (0)
#define P3OUT_INIT                                  (BIT0)

/*
    P4.0 = LCD segment line 35
    P4.1 = LCD segment line 34
    P4.2 = LCD segment line 33
    P4.3 = LCD segment line 32
    P4.4 = LCD segment line 31
    P4.5 = LCD segment line 30
    P4.6 = LCD segment line 29
    P4.7 = LCD segment line 28
 */
#define P4DIR_INIT                                  0
#define P4SEL_INIT                                  0
#define P4OUT_INIT                                  0

/*
    P5.0 = LCD segment line 27
    P5.1 = LCD segment line 26
    P5.2 = LCD segment line 25
    P5.3 = LCD segment line 24
    P5.4 = LCD segment line 23
    P5.5 = LCD segment line 22
    P5.6 = LCD segment line 21
    P5.7 = LCD segment line 20
*/
#define P5DIR_INIT                                  0
#define P5SEL_INIT                                  0
#define P5OUT_INIT                                  0

/*
    P6.0 = LCD segment line 19
    P6.1 = LCD segment line 18
    P6.2 = LCD segment line 17
    P6.3 = LCD segment line 16
    P6.4 = LCD segment line 15
    P6.5 = LCD segment line 14
    P6.6 = LCD segment line 13
    P6.7 = LCD segment line 12
*/
#define P5DIR_INIT                                  0
#define P5SEL_INIT                                  0
#define P5OUT_INIT                                  0

/*
    P7.0 = LCD segment line 11
    P7.1 = LCD segment line 10
    P7.2 = LCD segment line 9
    P7.3 = LCD segment line 8
    P7.4 = LCD segment line 7
    P7.5 = LCD segment line 6
    P7.6 = LCD segment line 5
    P7.7 = LCD segment line 4
 */
#define P7DIR_INIT                                  0
#define P7SEL_INIT                                  0
#define P7OUT_INIT                                  0

/*
    P8.0 = LCD segment line 3
    P8.1 = LCD segment line 2
    P8.2 = LCD segment line 1
    P8.3 = LCD segment line 0
    P8.4 = VREG EN
    P8.5 = Reset CC
    P8.6 = LED 1
    P8.7 = LED 2
 */
#define P8DIR_INIT                                  (BIT7 | BIT6)
#define P8SEL_INIT                                  0
#define P8OUT_INIT                                  (BIT7 | BIT6)


#endif

