#include "msp430.h"
#include <stdint.h>
#include "emeter-rtc-lib.h"
#include "emeter-rtc.h"

#if defined(ADC10_TEMP_MEASURE)

__no_init volatile unsigned short TEMP_H_ADC @ 0x1A20;  // 85
__no_init volatile unsigned short TEMP_L_ADC @ 0x1A1E;  // 30

static int32_t temperature_diode_slope;
static int32_t temperature_diode_intercept;


static uint8_t step_meas;

#define TEMP_H 			85L
#define TEMP_L 			30L

#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{
	if (lib_rtc_status & LIB_RTC_LOW_POWER)
	{
		switch (step_meas)
		{
			case 0:
				smooth_temperature(ADC10MEM0);
				if((lib_rtc_time.second & 0x07) == 0x07)
					rtc_temperature_compensation_estimate();					
				step_meas = 0;
				break;
			default:
				ADC10IFG &= ~ADC10IFG0; 
				adc10_init();				
				break;				
		}
	} else {
		switch (step_meas)
		{
			case 0:
				smooth_temperature(ADC10MEM0);
				ADC10MCTL0 = ADC10SREF_1 | BATT_ADC_CHANNEL; 	// Vref+ and Avss, Battery	
				if((lib_rtc_time.second & 0x07) == 0x07)
					rtc_temperature_compensation_estimate();						
				step_meas = 1;
				ADC10CTL0 |= ADC10SC;  						// start conversion
				break;
			case 1:
				smooth_battery_adc(ADC10MEM0);
				step_meas = 0;
				break;	
			default:
				ADC10IFG &= ~ADC10IFG0; 
				adc10_init();				
				break;				
		}
	}  
}

// 1/ 2^4  unit
int16_t temperature_measurement(void)
{
	return (raw_temperature_from_adc*temperature_diode_slope + temperature_diode_intercept) >> 12;
}

void temperature_coeff_setting(void)
{
	if (TEMP_H_ADC - TEMP_L_ADC)
	{
		temperature_diode_slope = (((TEMP_H - TEMP_L)<< 16)/(TEMP_H_ADC - TEMP_L_ADC)) >> 3;
		temperature_diode_intercept = ((TEMP_H_ADC*TEMP_L - TEMP_L_ADC*TEMP_H) << 16)/(TEMP_H_ADC - TEMP_L_ADC);
	}
}

// Todo : If use the DMA for ADC, the power will be lower than irq mode  
void adc10_init(void)
{
	// Setup internal reference
	while (REFCTL0 & REFGENBUSY) ;						// If ref generator busy, WAIT
	REFCTL0 = REFON | REFVSEL_1 | REFMSTR;		// Select internal ref = 2.0V
																						// Internal Reference ON
  
  ADC10CTL0 = ADC10SHT_8 | ADC10ON; 				//256 cycles S&H, switch on ADC10
  // ADC10 start bit used for triggering, pulse mode, MODCLK, single chan single con
  ADC10CTL1 = ADC10SHP | ADC10DIV_0 | ADC10SSEL_0 | ADC10CONSEQ_0; 
  //10 bit resolution, 0~50 ksps, ref on only during S&H
  ADC10CTL2 = ADC10PDIV_0 | ADC10RES | ADC10SR | ADC10REFBURST; 
  ADC10MCTL0 = ADC10SREF_1 | ADC10INCH_10;  //Vref+ and Avss, temp sensor, 
  
	step_meas = 0;  
  ADC10IE = ADC10IE0; 											// enable coversion IE
  //while (ADC10CTL1 & ADC10BUSY) ; 
  ADC10CTL0 |= ADC10ENC;  									// Prepare conversion

}


#endif


