//******************************************************************************
//  MSP430F6736 RTC Calibration Library for China Smart Grid
// 	China MCU SAE Team.
//  Texas Instruments Inc.
//******************************* **********************************************

#include "msp430.h"
#include <stdint.h>
#include <stdlib.h>
#include "emeter-rtc-inter.h"
#include "emeter-rtc-lib.h"
#include "emeter-rtc.h"
#include "emeter-flash.h"

#define STEP_BASE_OFFSET_MODE		1

void set_rtc_parameter(int address, int32_t value)
{
  //int16_t *addr16;

	switch (address)
	{ 
//------------------------------------------------------------------------		
	case RTC_CRYSTAL_COEFF0:				// K coeff0
		break;
	case RTC_CRYSTAL_COEFF1:				// K coeff1
		break;	
	case RTC_CRYSTAL_COEFF2:				// K coeff2
		break;	
	case RTC_CRYSTAL_COEFF3:				// K coeff3
		break;	
	case RTC_CRYSTAL_COEFF4:				// K coeff4	
		break;	
	case RTC_CRYSTAL_COEFF5:				// K coeff5	
		break;			
//------------------------------------------------------------------------		
  case RTC_CRYSTAL_BASE_OFFSET:	// set/get RTC offset  
		RTCCTL0_H = RTCKEY_H; 								 // Unlock RTC_C module
#if STEP_BASE_OFFSET_MODE
		value = crystal_base_error + value;		 // + down, - up
#endif		
		if (value > 240)	 
			value = 240;
		if (value < -240)
			value = -240;

		if (value < 0)
			RTCOCAL = 0x8000 - value;
		else
			RTCOCAL = value;											

		RTCCTL0_H = 0;												 // Lock RTC_C module

#if LIB_INFO_COMBINE_EN
		struct info_mem_s nv_parms_dak;

		nv_parms_dak = nv_parms;
		nv_parms_dak.seg_a.s.crystal_offset_ppm = value;
		flash_memcpy((char*)&nv_parms, (char*)&nv_parms_dak, sizeof(struct info_mem_s));	  
#else
		flash_clr((int *)&info_rtc[0]);
		flash_write_int16((int *)&info_rtc[0], value);
#endif
    break;
//------------------------------------------------------------------------	    
  case RTC_CURRENT_TIME:				// set/get current time
    set_time_internal((struct rtc_interface *)value);
    break;		
//------------------------------------------------------------------------	    
  case RTC_1S_INTR_CB: 					// set/get 1s interrupt callback
    rtc_intr_callback = (user_callback)value;
    break;	
//------------------------------------------------------------------------	    
  case RTC_ERROR_STATUS:
    lib_rtc_status &= ~LIB_RTC_ERROR;
  	break;
//------------------------------------------------------------------------	  	
  case RTC_NTC_POWER_ON_CB:
    open_pin_supply = (user_callback)value;			
    break;
//------------------------------------------------------------------------	    
  case RTC_NTC_POWER_OFF_CB:
    close_pin_supply = (user_callback)value;		  
    break;  
//------------------------------------------------------------------------	    
	}
}

int32_t get_rtc_parameter(int address, void *ptr)
{
	switch (address)
	{  
  case RTC_CRYSTAL_BASE_OFFSET:	// set/get RTC offset
    return crystal_base_error;
	case RTC_CURRENT_TEMP:
		return temperature_in_celsius; 
  case RTC_CURRENT_TIME:				// set/get current time
    get_time_internal(ptr);
    break;	
  case RTC_CURRENT_MODE:				// set/get current status
    if (lib_rtc_status & LIB_RTC_LOW_POWER)
      return RTC_LOW_POWER_STATUS;
    if (lib_rtc_status & LIB_RTC_OPEN)
      return RTC_OPEN_STATUS;
    return RTC_CLOSE_STATUS;
  case RTC_ERROR_STATUS:
    return ((lib_rtc_status & LIB_RTC_ERROR)? RTC_TRUE:RTC_FALSE);		
  // 2.5V ref for NTC / 2.0V ref for internal sensor
  case RTC_ADC_BATTERY_RAW:
  	return raw_battery_from_adc;
	}
  return 0;
}

