//******************************************************************************
//  MSP430F6736 RTC Calibration Library for China Smart Grid
// 	China MCU SAE Team.
//  Texas Instruments Inc.
//******************************* **********************************************

#include "emeter_global_config.h"

#ifndef __EMETER_RTC_INTER_H__
#define __EMETER_RTC_INTER_H__

// Define the default time for emeter
#define EN_LIB_RESTORE_DEFAULT		0		// 0 :Disable 1:Enable 
// Time use the 1000 years count
#define EN_THOUSAND_TIME 		0		// 0: disable 1: enable
// Use the BCD for time 
#define EN_RTC_BCD					0		// 0: disable 1: enable
// Use time1 or time2 for 1s pulse
#define EN_TA2_PWM					0		// 0: time1 using 1: time2 using

#if 0
// Enable RTC and Algrithm library information together
#define LIB_INFO_COMBINE_EN	1		// 0: disable 1: enable
// Enable crystal coeff setting
#define CRYSTAL_QUADRATIC_COEFF_SETTING	1
#endif
//--------------------------------------------------------------------------------------------------------------------
// Can work together
#define SOFTWARE_RTC_SUPPORT
#undef SOFTWARE_RTC_SUPPORT
#define HARDWARE_RTC_SUPPORT
// Software time canot support BCD now.
#if defined(SOFTWARE_RTC_SUPPORT) && (EN_RTC_BCD == 1) 
#error Compiler not recognised.
#endif

//--------------------------------------------------------------------------------------------------------------------
// Can't work together
#define ADC10_TEMP_MEASURE
#undef ADC10_TEMP_MEASURE
#define NTC_TEMP_MEASURE
#undef NTC_TEMP_MEASURE
#define NTC_TEMP_MEASURE1
//#undef NTC_TEMP_MEASURE1

#if defined(ADC10_TEMP_MEASURE) && (defined(NTC_TEMP_MEASURE) || defined(NTC_TEMP_MEASURE1))
#error Internal and ntc temperature canot work together.
#endif

#if EN_LIB_RESTORE_DEFAULT
#if EN_RTC_BCD
#define DEFAULT_YEAR	0x2012
#define DEFAULT_MONTH	0x01
#define DEFAULT_WEEK	0x00
#define DEFAULT_DAY		0x01
#define DEFAULT_HOUR	0x00
#define DEFAULT_MIN		0x00
#define DEFAULT_SEC		0x00
#else
#define DEFAULT_YEAR	2012
#define DEFAULT_MONTH	1
#define DEFAULT_WEEK	0
#define DEFAULT_DAY		1
#define DEFAULT_HOUR	0
#define DEFAULT_MIN		0
#define DEFAULT_SEC		0
#endif
#endif

#if EN_THOUSAND_TIME
typedef union
{
  uint8_t byte[2];
  int16_t word;
} word_byte;
#endif

struct rtc_interface {
#if EN_THOUSAND_TIME	
	word_byte year;	
#else
	uint8_t year; 
#endif
	uint8_t month;
	uint8_t week;
	uint8_t day;	
	uint8_t hour;
	uint8_t minute;
	uint8_t second;	
};

//--------------------------------------------------------------------------------------------------------------------
enum RTC_PARA_TABLE {
	RTC_CRYSTAL_BASE_OFFSET = 0, 	// set/get RTC offset
	RTC_CURRENT_TIME,	 				// set/get current time
	RTC_CURRENT_MODE,					// get current mode
	RTC_CURRENT_TEMP,					// get current temperature (unit: 0.5)	
	RTC_ERROR_STATUS,					// set/get rtc work status
	RTC_1S_INTR_CB, 					// set 1s interrupt callback
	RTC_NTC_POWER_ON_CB, 			// set power on callback
	RTC_NTC_POWER_OFF_CB, 		// set power down callback			
	RTC_ADC_BATTERY_RAW,			// get raw adc value based on 2.5V reference
	RTC_CRYSTAL_COEFF0,				// K coeff0
	RTC_CRYSTAL_COEFF1,				// K coeff1
	RTC_CRYSTAL_COEFF2,				// K coeff2
	RTC_CRYSTAL_COEFF3,				// K coeff3
	RTC_CRYSTAL_COEFF4,				// K coeff4	
	RTC_CRYSTAL_COEFF5,				// K coeff5		
}; 

enum RTC_STATUS {
	RTC_LOW_POWER_STATUS = 0,
	RTC_OPEN_STATUS,
	RTC_CLOSE_STATUS,
}; 


//--------------------------------------------------------------------------------------------------------------------
/* These values come from the crystal's data sheet. The are characteristic of the crystal
design, and do not significantly vary from sample to sample. Crystals vary in a parabolic
manner with temperature. */

/* This value is the temperature, in 1/16 degrees C, of the parabolic error curve for the crystal. */
#define CRYSTAL_QUADRATIC_CENTRE_TEMPERATURE    368		//23C*16

/* This value must be 65536 times the parts per million per degree Celcius squared error specified
for the crystal. The following is for a -0.035ppm per C^2 crystal. */
//#define CRYSTAL_QUADRATIC_COEFF2                 2195  //0.0335ppm*65536 -0.0335ppm

// adc channel
#define NTC_ADC_CHANNEL					ADC10INCH_4
#define BATT_ADC_CHANNEL				ADC10INCH_3
#define DVCC_ADC_CHANNEL				ADC10INCH_12

//----------------------------------------interface for user----------------------------------------------------------
extern void set_rtc_parameter(int address, int32_t value);
// Input parameter two only for rtc time get
extern int32_t get_rtc_parameter(int address, void *ptr);
extern void rtc_lib_open(void);
extern void rtc_lib_close(void);
extern void rtc_lib_enter_lpm(void);
extern void rtc_init(void);

#endif

