//******************************************************************************
//  MSP430F6736 RTC Calibration Library for China Smart Grid
// 	
//  Texas Instruments Inc.
//******************************* ***********************************************



#ifndef __TI_RTC_LIB_H__
#define __TI_RTC_LIB_H__
#include "emeter-rtc-inter.h"


#define RTC_TRUE		1
#define RTC_FALSE		0

// The time occur error
#define LIB_RTC_ERROR					0x01
// The second pulse
#define LIB_RTC_1SPULSE       0x02
// Low power mode which donot have 1s pulse 
#define LIB_RTC_LOW_POWER     0x04
// RTC library open flag
#define LIB_RTC_OPEN     			0x08

//--------------------------------------------------------------------------------------------------------------------
// crystal base offset in ppm which need be calibrated 
// This is defined in measurement library
//(NDEBUG)

#if LIB_INFO_COMBINE_EN
#include "emeter-metrology-structs.h"
#define crystal_base_error		(nv_parms.seg_a.s.crystal_offset_ppm)
#if CRYSTAL_QUADRATIC_COEFF_SETTING
#define CRYSTAL_QUADRATIC_COEFF 	(&nv_parms.seg_a.s.crystal_coeff[0])
#endif
#else
#define __infomem__a_     _Pragma("location=\"INFOA\"")
__infomem__a_ extern const int16_t info_rtc[64];
#define crystal_base_error	info_rtc[0] 	
#if CRYSTAL_QUADRATIC_COEFF_SETTING
#define CRYSTAL_QUADRATIC_COEFF   (&info_rtc[1])    //2195  //0.0335ppm*65536 -0.0335ppm
#endif
#endif

//--------------------------------------------------------------------------------------------------------------------
extern int16_t raw_temperature_from_adc;           
extern int16_t temperature_in_celsius;
extern int16_t raw_battery_from_adc;
extern int16_t raw_dvcc_from_adc;

// status about rtc
extern uint8_t lib_rtc_status;

//--------------------------------------------------------------------------------------------------------------------
extern void smooth_temperature(int16_t temperature);
extern void smooth_battery_adc(int16_t battery_adc);
extern void smooth_DVCC_adc(int16_t dvcc_adc);
//extern int16_t smooth_temp_final(int16_t temperature, int16_t before);

extern void rtc_temperature_compensation_estimate(void);
extern int16_t temperature_measurement(void);
extern void temperature_coeff_setting(void);
// Todo : If use the DMA for ADC, the power will be lower than irq mode  
extern void adc10_init(void);

//--------------------------------------------------------------------------------------------------------------------
typedef void (*user_callback)(void);
//extern user_callback other_low_level_init;
extern user_callback rtc_intr_callback;
extern user_callback open_pin_supply;
extern user_callback close_pin_supply;
extern user_callback pulse_pin_setting;

#endif

