#include "msp430.h"
#include <stdint.h>
#include "emeter-rtc-lib.h"
#include "emeter-rtc.h"

#if defined(NTC_TEMP_MEASURE)

#include "emeter-rtc-ntc-table.h"

// Use AUX adc
#define USE_AUX_ADC		1

static uint8_t step_meas;


int find_position_16bit_reverse(uint32_t value, const uint16_t *array,int16_t high)
{
	int16_t low, mid,position;
		
  low = 0;
	while (low <= high)
	{
		mid = (low + high) >> 1;
		if (array[mid] == value)
		{
			position = mid;
			break;
		} else if (value < array[mid] )
			low = mid + 1;
		else
			high = mid - 1;		
	}
  
	if (low > high)
		position =  (low + high) >> 1;
  
	return position;		
}

int find_position_32bit_reverse(uint32_t value, const uint32_t  *array,int16_t high)
{
	int16_t low, mid, position;
		
  low = 0;
	while (low <= high)
	{
		mid = (low + high) >> 1;
		if (array[mid] == value)
		{
			position = mid;
			break;
		} else if (value < array[mid] )
			low = mid + 1;
		else
			high = mid - 1;		
	}
  
	if (low > high)
		position =  (low + high) >> 1;
  
	return position;		
}


#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{
	if (lib_rtc_status & LIB_RTC_LOW_POWER)
	{
		switch (step_meas)
		{
			case 0:
				smooth_DVCC_adc(ADC10MEM0);
				ADC10CTL0 &= ~ADC10ENC;  									// Prepare conversion
				ADC10MCTL0 = ADC10SREF_1 | NTC_ADC_CHANNEL; 	// Vref+ and Avss, NTC	
				step_meas = 1;
				open_pin_supply();
				ADC10CTL0 |= ADC10ENC;  									// Prepare conversion
				ADC10CTL0 |= ADC10SC;							// start conversion
				break;
			case 1:
				smooth_temperature(ADC10MEM0);		
				ADC10CTL0 &= ~ADC10ENC;  									// Prepare conversion
#if USE_AUX_ADC	  
				ADC10MCTL0 = ADC10SREF_1 | ADC10INCH_12;	// Vref+ and Avss, DVCC/3 
#else
				ADC10MCTL0 = ADC10SREF_1 | ADC10INCH_11;	// Vref+ and Avss, AVCC/2 maybe ok 
#endif			
				close_pin_supply();
				if((lib_rtc_time.second & 0x07) == 0x07)
					rtc_temperature_compensation_estimate();
				step_meas = 0;	
				break;
			default:
				ADC10IFG &= ~ADC10IFG0; 
				ADC10CTL0 &= ~ADC10ENC;  									// Prepare conversion
				adc10_init();				
				break;
		}
	} else {
		switch (step_meas)
		{
			case 0:
				smooth_DVCC_adc(ADC10MEM0);
				ADC10CTL0 &= ~ADC10ENC;  									// Prepare conversion
				ADC10MCTL0 = ADC10SREF_1 | NTC_ADC_CHANNEL; 	// Vref+ and Avss, NTC	
				step_meas = 1;
				open_pin_supply();
				ADC10CTL0 |= ADC10ENC;  									// Prepare conversion
				ADC10CTL0 |= ADC10SC;  						// start conversion
				break;
			case 1:
				smooth_temperature(ADC10MEM0);	
				ADC10CTL0 &= ~ADC10ENC;  									// Prepare conversion
				ADC10MCTL0 = ADC10SREF_1 | BATT_ADC_CHANNEL; 	// Vref+ and Avss, NTC					
				close_pin_supply();
				if((lib_rtc_time.second & 0x07) == 0x07)
					rtc_temperature_compensation_estimate();
				step_meas = 2;	
				ADC10CTL0 |= ADC10ENC;  									// Prepare conversion
				ADC10CTL0 |= ADC10SC;  						// start conversion				
				break;
			case 2:
				smooth_battery_adc(ADC10MEM0);
				ADC10CTL0 &= ~ADC10ENC;  									// Prepare conversion
#if USE_AUX_ADC	  
				ADC10MCTL0 = ADC10SREF_1 | ADC10INCH_12;	// Vref+ and Avss, DVCC/3 
#else
				ADC10MCTL0 = ADC10SREF_1 | ADC10INCH_11;	// Vref+ and Avss, AVCC/2 maybe ok 
#endif					
				step_meas = 0;
				break;
			default:
				ADC10IFG &= ~ADC10IFG0; 
				ADC10CTL0 &= ~ADC10ENC;  									// Prepare conversion
				adc10_init();
				break;				
		}
	}
}

// 1/ 2^4  unit
int16_t temperature_measurement(void)
{
	uint32_t temp;
	int16_t result;

	if (adc_start_flag == (TEMPERATURE_START|BATTERY_START|DVCC_START))
	{
#if USE_AUX_ADC	
	  temp = (((uint32_t)raw_temperature_from_adc) << 16)  / ((raw_dvcc_from_adc << 2) - raw_dvcc_from_adc)
#else
		temp = (((uint32_t)raw_temperature_from_adc) << 16)  / (raw_dvcc_from_adc << 1);
#endif
		temp = (temp * 25 * 1000 / (65536 - temp)) << 2; //0.001K
	  
		if (temp >= RT_TABLE_0D4C[sizeof(RT_TABLE_0D4C)/sizeof(uint32_t) - 1])
		{
			result = find_position_32bit_reverse(temp,RT_TABLE_0D4C,sizeof(RT_TABLE_0D4C)/sizeof(uint32_t)-1);
			result = ((-400 + (result<<2))<<4)/10;
		}
		else if (temp >= RT_TABLE_0D2C[sizeof(RT_TABLE_0D2C)/sizeof(uint16_t) - 1])	
		{
	  	result = find_position_16bit_reverse(temp,RT_TABLE_0D2C,sizeof(RT_TABLE_0D2C)/sizeof(uint16_t)-1);
			result = ((50 + (result<<1))<<4)/10;  	
	  }
	  else 
	  {
	  	result = find_position_16bit_reverse(temp,RT_TABLE_1C,sizeof(RT_TABLE_1C)/sizeof(uint16_t)-1);  
			result = ((36 + result)<<4) - 8;    	// adc accurate issue
	  }
  } else {
  	result = 400;		// 25*16
  }	
  return result;
}

//
void temperature_coeff_setting(void)
{

  
}


// Todo : If use the DMA for ADC, the power will be lower than irq mode  
void adc10_init(void)
{	
	// Setup internal reference
	while (REFCTL0 & REFGENBUSY) ;						// If ref generator busy, WAIT
	REFCTL0 = REFON | REFVSEL_2 | REFMSTR;		// Select internal ref = 2.5V
																						// Internal Reference ON
  ADC10CTL0 = ADC10SHT_8 | ADC10ON; 				// 256 cycles S&H, switch on ADC10
  // ADC10 start bit used for triggering, pulse mode, MODCLK, single chan single con
  ADC10CTL1 = ADC10SHP | ADC10DIV_0 | ADC10SSEL_0 | ADC10CONSEQ_0; 
  // 10 bit resolution, 0~50 ksps, ref on only during S&H
  ADC10CTL2 = ADC10PDIV_0 | ADC10RES | ADC10SR | ADC10REFBURST; 
  
#if USE_AUX_ADC	  
  AUXADCCTL |= AUXADC;		// AUX adc module
  ADC10MCTL0 = ADC10SREF_1 | ADC10INCH_12;  // Vref+ and Avss, DVCC/3 
#else
	ADC10MCTL0 = ADC10SREF_1 | ADC10INCH_11;	// Vref+ and Avss, AVCC/2 maybe ok 
#endif
	step_meas = 0;  
  ADC10IE = ADC10IE0; 											// enable coversion IE
  //while (ADC10CTL1 & ADC10BUSY) ; 
  //ADC10CTL0 |= ADC10ENC;  									// Prepare conversion
}

#endif

