//******************************************************************************
//  MSP430F6736 RTC Calibration Library for China Smart Grid
// 	China MCU SAE Team.
//  Texas Instruments Inc.
//******************************* **********************************************

#include "msp430.h"
#include <stdint.h>
#include "emeter-rtc.h"
#include "emeter-rtc-lib.h"

//------------------------------------------------------------------------------------------------
__no_init struct rtc_s	lib_rtc_time;
//------------------------------------------------------------------------------------------------
extern uint8_t lib_rtc_status;

//------------------------------------------------------------------------------------------------
void set_rtc_year(int year)
{
	RTCYEAR = year;
}

void set_rtc_mon(int month)
{
	RTCMON = month;
}

void set_rtc_day(int day)
{
	RTCDAY = day;
}

void set_rtc_dow(int dow)
{
	RTCDOW = dow;
}

void set_rtc_hour(int hour)
{
	RTCHOUR = hour;
}

void set_rtc_min(int min)
{
	RTCMIN = min;
}

void set_rtc_sec(int sec)
{
	RTCSEC = sec;
} 	

int get_rtc_year(void)
{
	return RTCYEAR;
}

int get_rtc_mon(void)
{
	return RTCMON;
}

int get_rtc_day(void)
{
	return RTCDAY;
}

int get_rtc_dow(void)
{
	return RTCDOW;
}

int get_rtc_hour(void)
{
	return RTCHOUR;
}

int get_rtc_min(void)
{
	return RTCMIN;
}

int get_rtc_sec(void)
{
	return RTCSEC;
}

void set_rtc_checksum(void)
{
  uint8_t chksum;  
	chksum	= lib_rtc_time.second;
	chksum += lib_rtc_time.minute;
	chksum += lib_rtc_time.hour;
	chksum += lib_rtc_time.day;			
	chksum += lib_rtc_time.week;	
	chksum += lib_rtc_time.month;	
#if EN_THOUSAND_TIME	
	chksum += lib_rtc_time.year.byte[0];  
	chksum += lib_rtc_time.year.byte[1]; 
#else
	chksum += lib_rtc_time.year;	
#endif
	lib_rtc_time.chksum = ~chksum;
}

int check_rtc_checksum(void)
{
  uint8_t chksum;  
  
  chksum	= lib_rtc_time.second;
  chksum += lib_rtc_time.minute;
  chksum += lib_rtc_time.hour;
  chksum += lib_rtc_time.day; 		
  chksum += lib_rtc_time.week;	
  chksum += lib_rtc_time.month; 
#if EN_THOUSAND_TIME	
  chksum += lib_rtc_time.year.byte[0];	
  chksum += lib_rtc_time.year.byte[1];			
#else
  chksum += lib_rtc_time.year;		
#endif
 
  chksum = ~chksum;
  return ((chksum == lib_rtc_time.chksum)? RTC_TRUE:RTC_FALSE); 
}  

// Don't check the time availability
void set_time_internal(struct rtc_interface *ptr_rtc)
{
  istate_t state; 
  
#if EN_THOUSAND_TIME	
	lib_rtc_time.year.word = ptr_rtc->year; 		
#else
	lib_rtc_time.year = ptr_rtc->year;		
#endif
	lib_rtc_time.month = ptr_rtc->month;			
	lib_rtc_time.week = ptr_rtc->week;		
	lib_rtc_time.day = ptr_rtc->day;	
	lib_rtc_time.hour = ptr_rtc->hour;	
	lib_rtc_time.minute = ptr_rtc->minute;	
	lib_rtc_time.second = ptr_rtc->second;	
	set_rtc_checksum();
	
	state = __get_interrupt_state();
	__disable_interrupt();				

	RTCCTL0_H = RTCKEY_H; 								 // Unlock RTC_C module	
#if EN_THOUSAND_TIME				
	set_rtc_year(lib_rtc_time.year.word);
#else
#if EN_RTC_BCD
	set_rtc_year(lib_rtc_time.year + 0x2000);
#else
	set_rtc_year(lib_rtc_time.year + 2000);
#endif
#endif
	
	set_rtc_mon(lib_rtc_time.month); 
	set_rtc_day(lib_rtc_time.day);
	set_rtc_dow(lib_rtc_time.week);
	set_rtc_hour(lib_rtc_time.hour);
	set_rtc_min(lib_rtc_time.minute);
	set_rtc_sec(lib_rtc_time.second);
	RTCCTL0_H = 0;												 // Lock RTC_C module
		
	__set_interrupt_state(state); 
}

void get_time_internal(struct rtc_interface *ptr_rtc)
{
  istate_t state; 
  
	state = __get_interrupt_state();
	__disable_interrupt();				
  
#if EN_THOUSAND_TIME	
	ptr_rtc->year.word = lib_rtc_time.year.word; 		
#else
	ptr_rtc->year =	lib_rtc_time.year;		
#endif
	ptr_rtc->month = lib_rtc_time.month; 
	ptr_rtc->day = lib_rtc_time.day;
	ptr_rtc->week = lib_rtc_time.week;
	ptr_rtc->hour = lib_rtc_time.hour;
	ptr_rtc->minute = lib_rtc_time.minute;
	ptr_rtc->second = lib_rtc_time.second;
  
	__set_interrupt_state(state); 
}

//------------------------------------------------------------------------------------------------
// When power on, the RTC need reset and restart
void rtc_init(void)
{
	// Must have clean since it will be called before init glabal var 
	lib_rtc_status = 0;
		
#ifdef HARDWARE_RTC_SUPPORT  	
	// Configure RTC_C
	RTCCTL0_H = RTCKEY_H;									 // Unlock RTC_C module
	RTCCTL0_L |= RTCRDYIE; 								 // Enable RTC read ready interrupt		
	
#if EN_RTC_BCD	
	RTCCTL1 |= RTCBCD | RTCHOLD; 					 // RTC enable BCD mode, RTC hold
#else
	RTCCTL1 |= RTCHOLD;					 					// RTC hold
#endif
	if (check_rtc_checksum())
	{
#if 0	
#if EN_THOUSAND_TIME	
		set_rtc_year(lib_rtc_time.year.word);
#else
		set_rtc_year(lib_rtc_time.year);
#endif
		set_rtc_mon(lib_rtc_time.month);	
		set_rtc_day(lib_rtc_time.day);
		set_rtc_dow(lib_rtc_time.week);
		set_rtc_hour(lib_rtc_time.hour);
		set_rtc_min(lib_rtc_time.minute);
		set_rtc_sec(lib_rtc_time.second);	
		//lib_rtc_status &= ~LIB_RTC_ERROR;
#endif		
	} else {
		lib_rtc_status |= LIB_RTC_ERROR;
    
#if EN_LIB_RESTORE_DEFAULT		
		set_rtc_year(DEFAULT_YEAR);
		set_rtc_mon(DEFAULT_MONTH);	
		set_rtc_day(DEFAULT_DAY);
		set_rtc_dow(DEFAULT_WEEK);
		set_rtc_hour(DEFAULT_HOUR);
		set_rtc_min(DEFAULT_MIN);
		set_rtc_sec(DEFAULT_SEC);
#endif		
	}
  
	RTCCTL1 &= ~(RTCHOLD); 								 // Start RTC calendar mode
	
  if(crystal_base_error < 0)
    RTCOCAL = 0x8000 - crystal_base_error; // up -
  else
    RTCOCAL = crystal_base_error;					// Down +
	
	RTCCTL0_H = 0; 												 // Lock RTC_C module
#endif	
}


#ifdef HARDWARE_RTC_SUPPORT  
#pragma vector=RTC_VECTOR
__interrupt void RTC_ISR(void)
{
  switch (__even_in_range(RTCIV, 16))
  {
  case RTCIV_NONE:                    // No interrupts
    break;
  case RTCIV_RTCOFIFG:                // RTCOFIFG
    break;
  case RTCIV_RTCRDYIFG:               // RTCRDYIFG       
    if (!check_rtc_checksum())
      lib_rtc_status |= LIB_RTC_ERROR;
    lib_rtc_time.second = get_rtc_sec();
    lib_rtc_time.minute = get_rtc_min();
    lib_rtc_time.hour = get_rtc_hour(); 		
    lib_rtc_time.day = get_rtc_day();
    lib_rtc_time.week = get_rtc_dow();
    lib_rtc_time.month = get_rtc_mon();
#if EN_THOUSAND_TIME						
    lib_rtc_time.year.word = get_rtc_year();
#else
#if EN_RTC_BCD
    lib_rtc_time.year = get_rtc_year();
#else
    lib_rtc_time.year = get_rtc_year() - 2000;
#endif
#endif
    set_rtc_checksum();							// Every second reading         
    rtc_intr_callback();

    if (lib_rtc_status & LIB_RTC_OPEN)
    {
	    //start temperature conversion
	    ADC10CTL0 |= ADC10ENC;  				// Prepare conversion
	    ADC10CTL0 |= ADC10SC;
    }
    break;
  case RTCIV_RTCTEVIFG:               // RTCEVIFG
    __no_operation();               // Interrupts every minute
    break;
  case RTCIV_RTCAIFG:                 // RTCAIFG
    __no_operation();               // Interrupts every alarm event
    break;
  case RTCIV_RT0PSIFG:                // RT0PSIFG
    break;
  case RTCIV_RT1PSIFG:                // RT1PSIFG
    break;
  case 14: break;                     // Reserved
  case 16: break;                     // Reserved
  default: break;
  }
}
#endif

#if defined(SOFTWARE_RTC_SUPPORT)

/* Return values for bump_rtc. These indicate the most significant element
of the rtc which was changed. The rtc is not updated if it's sumcheck is
incorrect, so nothing is updated for an inconsistent rtc setting. */
#define RTC_INCONSISTENT		0
#define RTC_CHANGED_SECOND	1
#define RTC_CHANGED_MINUTE	2
#define RTC_CHANGED_HOUR		3
#define RTC_CHANGED_DAY			4
#define RTC_CHANGED_MONTH		5
#define RTC_CHANGED_YEAR		6

const int8_t month_lengths[13] =
{
  00,
  31, 28, 31,
  30, 31, 30,
  31, 31, 30,
  31, 30, 31
};

/* Bump the RTC by a second, and return an indication of the most significant
element of the date to have changed. Maintain a sumcheck byte to help check
for corrupted RTC settings after a reset. */
int bump_rtc(void)
{
  /* First check the consistency of the current rtc setting. If it is inconsistent,
  (i.e. has a bad sumcheck) do not bump it. */
  if (!check_rtc_sumcheck())
    return RTC_INCONSISTENT;
  if (++lib_rtc_time.second < 60)
  {
    set_rtc_checksum();
    return RTC_CHANGED_SECOND;
  }
  lib_rtc_time.second = 0;
  if (++lib_rtc_time.minute < 60)
  {
    set_rtc_checksum();
    return RTC_CHANGED_MINUTE;
  }
  lib_rtc_time.minute = 0;
  if (++lib_rtc_time.hour < 24)
  {
    set_rtc_checksum();
    return RTC_CHANGED_HOUR;
  }
  lib_rtc_time.hour = 0;
  if ((lib_rtc_time.month == 2  &&  (lib_rtc_time.year & 3) == 0  &&  lib_rtc_time.day < 29)
      ||
        lib_rtc_time.day < month_lengths[lib_rtc_time.month])
  {
    ++lib_rtc_time.day;
    set_rtc_checksum();
    return RTC_CHANGED_DAY;
  }
  lib_rtc_time.day = 1;
  if (++lib_rtc_time.month <= 12)
  {
    set_rtc_checksum();
    return RTC_CHANGED_MONTH;
  }
  lib_rtc_time.month = 1;
  ++lib_rtc_time.year;
  set_rtc_checksum();
  return RTC_CHANGED_YEAR;
}

int leap_year(void)
{
  /* This simple test is good enough for this century */
  return (lib_rtc_time.year & 0x03) == 0;
}

/* Return the current day of the week, based on the RTC. */
int weekday(void)
{
  int i;
  int days;
  
  /* This works for years 2000-2099 */
  /* Result is 0=Sunday, 1=Monday, etc. */
  /* Allow for the day of the month */
  days = lib_rtc_time.day + 6;
  /* Allow for the months to date this year... */
  for (i = 1;  i < lib_rtc_time.month;  i++)
    days += month_lengths[i];
  /* ...with a little offset if we are early in a leap year */
  if ((lib_rtc_time.year & 0x03) == 0  &&  lib_rtc_time.month <= 2)
    days--;
  /* Allow for the years... */
  days += lib_rtc_time.year;
  /* ...and a little extra for the leap years */
  days += (lib_rtc_time.year >> 2);
  days %= 7;
  return days;
}

/* Perform a temperature compensated one second update of the RTC */
int update_rtc(void)
{
  integrated_rtc_correction += current_rtc_correction;
  if (integrated_rtc_correction >= CRYSTAL_BASE_CYCLES_PER_SECOND)
  {
    integrated_rtc_correction -= CRYSTAL_BASE_CYCLES_PER_SECOND;
    /* We need to add an extra second to the RTC */
    bump_rtc();
  }
  else if (integrated_rtc_correction <= -CRYSTAL_BASE_CYCLES_PER_SECOND)
  {
    integrated_rtc_correction += CRYSTAL_BASE_CYCLES_PER_SECOND;
    /* We need to drop a second from the RTC */
    return 0;
  }
  bump_rtc();
  
  return 1;
}
#endif




